﻿// Copyright (C) 2003, 2005 Daisuke Arai <darai@users.sourceforge.jp>
// Copyright (C) 2013 panacoran <panacoran@users.sourceforge.jp>
// 
// This program is part of Protra.
//
// Protra is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/>.
// 
// $Id: IfStatementNode.cs 455 2013-06-08 11:52:33Z panacoran $

using System.Collections.Generic;

namespace Protra.Lib.Lang
{
    /// <summary>
    /// if文のノードを表すクラス。
    /// </summary>
    public class IfStatementNode : Node
    {
        /// <summary>
        /// 条件式のノード
        /// </summary>
        private Node _condNode;

        /// <summary>
        /// ノードのリスト
        /// </summary>
        private readonly List<Node> _nodeList = new List<Node>();

        /// <summary>
        /// 次のelsif句またはelse句
        /// </summary>
        private IfStatementNode _nextNode;

        /// <summary>
        /// 構文解析する。
        /// </summary>
        /// <exception cref="Protra.Lib.Lang.ParseException">
        /// 構文解析中にエラーが発生したときにthrowされる。
        /// </exception>
        /// <return>
        /// 解析した結果のノード。
        /// </return>
        public override Node Parse()
        {
            Token = Scanner.Token;

            if (!Scanner.Scan())
                throw new ParseException("expression is expected", Token);
            _condNode = new ExpressionNode().Parse();
            if (Scanner.Token.Value != ";")
                throw new ParseException("';' expected", Token);

            while (Scanner.Scan())
            {
                var token = Scanner.Token.Value;
                if (token == "end")
                {
                    Scanner.Scan();
                    if (Scanner.Token.Value != ";")
                        throw new ParseException("';' expected", Scanner.Token);
                    return this;
                }
                if (token == "elsif")
                {
                    _nextNode = (IfStatementNode)(new IfStatementNode().Parse());
                    return this;
                }
                if (token == "else")
                {
                    _nextNode = ParseElse();
                    return this;
                }
                _nodeList.Add(new StatementNode().Parse());
            }
            throw new ParseException("unexpected EOF", Token);
        }

        private IfStatementNode ParseElse()
        {
            Scanner.Scan();
            if (Scanner.Token.Value != ";")
                throw new ParseException("';' expected", Scanner.Token);

            var node = new IfStatementNode {_condNode = new LiteralNode(new Value(true))};
            while (Scanner.Scan())
            {
                if (Scanner.Token.Value == "end")
                {
                    Scanner.Scan();
                    if (Scanner.Token.Value != ";")
                        throw new ParseException("';' expected", Scanner.Token);
                    return node;
                }
                node._nodeList.Add(new StatementNode().Parse());
            }
            throw new ParseException("unexpected EOF", Token);
        }

        /// <summary>
        /// プログラムを実行する。
        /// </summary>
        /// <exception cref="Protra.Lib.Lang.RuntimeException">
        /// プログラム実行中にエラーが発生した場合にthrowされる。
        /// </exception>
        /// <param name="resource">リソース</param>
        /// <param name="at">int型@作用素の値</param>
        /// <param name="ats">string型@作用素の値</param>
        /// <returns>実行結果</returns>
        public override Value Execute(Resource resource, int at, string ats)
        {
            var val = _condNode.Execute(resource, at, ats);
            if (val != null && val.IsTrue)
                foreach (var node in _nodeList)
                    node.Execute(resource, at, ats);
            else if (_nextNode != null)
                _nextNode.Execute(resource, at, ats);
            return null;
        }
    }
}